@@ -1,11 +1,11 @@
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
 %  Integration code for SWI Prolog 6.6.0 and later versions to compile and
 %  load Logtalk files using SWI Prolog consult/1, to support edit/1 and
 %  make/0, and to improve usability when using the XPCE profiler and XPCE
 %  graphical debugger
 %
-%  Last updated on October 27, 2015
+%  Last updated on February 14, 2016
 %
 %  This file is part of Logtalk <http://logtalk.org/>  
 %  Copyright 1998-2015 Paulo Moura <pmoura@logtalk.org>
@@ -25,30 +25,36 @@
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
 
-
 :- multifile(user:prolog_load_file/2).
 :- dynamic(user:prolog_load_file/2).
 
 user:prolog_load_file(_:Spec, Flags) :-
-	\+ '$lgt_member'(must_be_module(true), Flags),	% exclude calls to use_module/1-2
-	\+ '$lgt_member'(if(not_loaded), Flags),			% exclude calls to ensure_loaded/1
-	\+ absolute_file_name(Spec, [extensions([pl]), access(read), file_errors(fail)], _),
+	% exclude calls to use_module/1-2
+	\+ '$lgt_member'(must_be_module(true), Flags),
+	% exclude calls to ensure_loaded/1
+	\+ '$lgt_member'(if(not_loaded), Flags),
+	findall(PrologExtension, user:prolog_file_type(PrologExtension,prolog), PrologExtensions),
+	\+ absolute_file_name(Spec, [extensions(PrologExtensions), access(read), file_errors(fail)], _),
+	findall(
+		LogtalkExtension,
+		(	'$lgt_file_extension'(logtalk, DotLogtalkExtension),
+			atom_concat('.', LogtalkExtension, DotLogtalkExtension)
+		),
+		LogtalkExtensions
+	),
 	(	atom(Spec) ->
 		expand_file_name(Spec, [SpecExp]),
-		absolute_file_name(SpecExp, [extensions([lgt,logtalk]), access(read), file_errors(fail)], Path)
+		absolute_file_name(SpecExp, [extensions(LogtalkExtensions), access(read), file_errors(fail)], Path)
 	;	Spec =.. [Library, File],
-		atom(File),										% no paths instead of a file name for Logtalk
+		% no paths instead of a file name for Logtalk
+		atom(File),
 		'$lgt_expand_library_path'(Library, LibPath),
 		atom_concat(LibPath, File, Spec2),
 		expand_file_name(Spec2, [SpecExp]),
-		absolute_file_name(SpecExp, [extensions([lgt,logtalk]), access(read), file_errors(fail)], Path)
+		absolute_file_name(SpecExp, [extensions(LogtalkExtensions), access(read), file_errors(fail)], Path)
 	),
-	file_directory_name(Path, Dir),
-	file_base_name(Path, BaseName),
-	file_name_extension(Entity, _, BaseName),
-	working_directory(Old, Dir),
 	'$lgt_swi_filter_compiler_flags'(Flags, Flags2),
-	setup_call_cleanup(true, logtalk_load(Entity, Flags2), working_directory(_, Old)).
+	logtalk_load(Path, Flags2).
 
 
 '$lgt_swi_filter_compiler_flags'([], []).
@@ -65,129 +71,35 @@ user:prolog_load_file(_:Spec, Flags) :-
 
 :- multifile(prolog_edit:locate/3).
 
-prolog_edit:locate(Name, source_file(Source), [file(Source)]) :-
-	atom(Name),
-	file_name_extension(Name, 'pl', PrologFile),
-	source_file(Path),
-	file_base_name(Path, PrologFile),
-	source_file_property(Path, derived_from(Source,_)),
-	file_base_name(Source, LogtalkFile),
-	(	file_name_extension(Name, 'lgt', LogtalkFile)
-	;	file_name_extension(Name, 'logtalk', LogtalkFile)
-	),
-	!.
-
 prolog_edit:locate(Spec, source_file(Source), [file(Source)]) :-
 	compound(Spec),
 	Spec =.. [Library, Name],
 	atom(Name),
-	file_name_extension(Name, 'pl', PrologFile),
-	source_file(Path),
-	file_base_name(Path, PrologFile),
-	source_file_property(Path, derived_from(Source,_)),
-	file_base_name(Source, LogtalkFile),
-	(	file_name_extension(Name, 'lgt', LogtalkFile)
-	;	file_name_extension(Name, 'logtalk', LogtalkFile)
-	),
 	'$lgt_expand_library_path'(Library, LibraryPath),
-	atom_concat(LibraryPath, LogtalkFile, Source),
+	atom_concat(LibraryPath, Name, LogtalkPath),
+	(	file_name_extension(_, Extension, LogtalkPath),
+		atom_concat('.', Extension, DotExtension),
+		'$lgt_file_extension'(logtalk, DotExtension) ->
+		Source = LogtalkPath
+	;	'$lgt_file_extension'(logtalk, DotExtension),
+		atom_concat(LogtalkPath, DotExtension, Source)
+	),
+	source_file_property(_, derived_from(Source,_)),
 	!.
 
 
-:- multifile(user:prolog_predicate_name/2).	% for e.g. the call stack in the SWI-Prolog graphical tracer
+% for e.g. the call stack in the SWI-Prolog graphical tracer
+:- multifile(user:prolog_predicate_name/2).
 
-user:prolog_predicate_name(user:'$lgt_send_to_obj_'(_, _, _), '::/2 (event-aware)') :- !.
-user:prolog_predicate_name(user:'$lgt_send_to_obj_ne_'(_, _, _), '::/2 (event transparent)') :- !.
-user:prolog_predicate_name(user:'$lgt_send_to_self_'(_, _, _), '::/1') :- !.
-user:prolog_predicate_name(user:'$lgt_obj_super_call_'(_, _, _), '^^/2 (from obj; same pred)') :- !.
-user:prolog_predicate_name(user:'$lgt_ctg_super_call_'(_, _, _), '^^/2 (from ctg; same pred)') :- !.
-user:prolog_predicate_name(user:'$lgt_call_in_this'(_, _), 'call/1') :- !.
-
-user:prolog_predicate_name(user:'$lgt_send_to_obj_rt'(_, _, _, _), '::/2 (runtime)') :- !.
-user:prolog_predicate_name(user:'$lgt_send_to_obj'(_, _, _), '::/2 (event transparent)') :- !.
-user:prolog_predicate_name(user:'$lgt_send_to_obj_ne'(_, _, _), '::/2 (event transparent)') :- !.
-user:prolog_predicate_name(user:'$lgt_send_to_self'(_, _, _), '::/1') :- !.
-user:prolog_predicate_name(user:'$lgt_obj_super_call'(_, _, _), '^^/2 (from obj; same pred)') :- !.
-user:prolog_predicate_name(user:'$lgt_ctg_super_call'(_, _, _), '^^/2 (from ctg; same pred)') :- !.
-
-user:prolog_predicate_name(user:'$lgt_metacall'(_, _, _), 'call/N') :- !.
-user:prolog_predicate_name(user:'$lgt_metacall'(_, _), 'call/1') :- !.
-user:prolog_predicate_name(user:'$lgt_quantified_metacall'(_, _, _), 'call/1') :- !.
-user:prolog_predicate_name(user:'$lgt_metacall_local'(_, _), 'call/1') :- !.
-user:prolog_predicate_name(user:'$lgt_metacall_sender'(_, _, _, _), 'call/1') :- !.
-
-user:prolog_predicate_name(user:'$lgt_bagof'(_, _, _, _, _), 'bagof/3') :- !.
-user:prolog_predicate_name(user:'$lgt_setof'(_, _, _, _, _), 'setof/3') :- !.
-
-user:prolog_predicate_name(user:'$lgt_expand_term_local'(_, _, _, _), 'expand_term/2') :- !.
-user:prolog_predicate_name(user:'$lgt_expand_term_message'(_, _, _, _, _), 'expand_term/2') :- !.
-user:prolog_predicate_name(user:'$lgt_expand_goal_local'(_, _, _, _), 'expand_goal/2') :- !.
-user:prolog_predicate_name(user:'$lgt_expand_goal_message'(_, _, _, _, _), 'expand_goal/2') :- !.
-
-user:prolog_predicate_name(user:'$lgt_phrase'(_, _, _), 'phrase/2') :- !.
-user:prolog_predicate_name(user:'$lgt_phrase'(_, _, _, _), 'phrase/3') :- !.
-
-user:prolog_predicate_name(user:'$lgt_compiler_flag'(_, _), 'current_logtalk_flag/2') :- !.
-user:prolog_predicate_name(user:'$lgt_set_compiler_flag'(_, _), 'set_logtalk_flag/2') :- !.
-
-user:prolog_predicate_name(user:'$lgt_current_op'(_, _, _, _, _, _), 'current_op/3') :- !.
-user:prolog_predicate_name(user:'$lgt_current_predicate'(_, _, _, _), 'current_predicate/1') :- !.
-user:prolog_predicate_name(user:'$lgt_predicate_property'(_, _, _, _, _), 'predicate_property/2') :- !.
-
-user:prolog_predicate_name(user:'$lgt_abolish_checked'(_, _, _, _), 'abolish/1') :- !.
-user:prolog_predicate_name(user:'$lgt_asserta_fact_checked'(_, _, _, _, _), 'asserta/1') :- !.
-user:prolog_predicate_name(user:'$lgt_asserta_rule_checked'(_, _, _, _, _), 'asserta/1') :- !.
-user:prolog_predicate_name(user:'$lgt_assertz_fact_checked'(_, _, _, _, _), 'assertz/1') :- !.
-user:prolog_predicate_name(user:'$lgt_assertz_rule_checked'(_, _, _, _, _), 'assertz/1') :- !.
-user:prolog_predicate_name(user:'$lgt_clause_checked'(_, _, _, _, _), 'clause/2') :- !.
-user:prolog_predicate_name(user:'$lgt_retract_fact_checked'(_, _, _, _), 'retract/1') :- !.
-user:prolog_predicate_name(user:'$lgt_retract_rule_checked'(_, _, _, _), 'retract/1') :- !.
-user:prolog_predicate_name(user:'$lgt_retractall_checked'(_, _, _, _), 'retractall/1') :- !.
-
-user:prolog_predicate_name(user:'$lgt_iso_read_term'(_, _, _, _), 'read_term/3') :- !.
-user:prolog_predicate_name(user:'$lgt_iso_read_term'(_, _, _), 'read_term/2') :- !.
-user:prolog_predicate_name(user:'$lgt_iso_read'(_, _, _), 'read/2') :- !.
-user:prolog_predicate_name(user:'$lgt_iso_read'(_, _), 'read/1') :- !.
-
-user:prolog_predicate_name(user:'$lgt_iso_write_term'(_, _, _, _), 'write_term/3') :- !.
-user:prolog_predicate_name(user:'$lgt_iso_write_term'(_, _, _), 'write_term/2') :- !.
-user:prolog_predicate_name(user:'$lgt_iso_write'(_, _, _), 'write/2') :- !.
-user:prolog_predicate_name(user:'$lgt_iso_write'(_, _), 'write/1') :- !.
-user:prolog_predicate_name(user:'$lgt_iso_writeq'(_, _, _), 'writeq/2') :- !.
-user:prolog_predicate_name(user:'$lgt_iso_writeq'(_, _), 'writeq/1') :- !.
-
-user:prolog_predicate_name(user:'$lgt_category_parameter'(_, _, _, _), 'parameter/2') :- !.
-
-user:prolog_predicate_name(user:'$lgt_threaded_or'(_, _, _), 'threaded/1') :- !.
-user:prolog_predicate_name(user:'$lgt_threaded_and'(_, _, _), 'threaded/1') :- !.
-user:prolog_predicate_name(user:'$lgt_threaded_ignore'(_), 'threaded_ignore/1') :- !.
-user:prolog_predicate_name(user:'$lgt_threaded_call'(_, _, _), 'threaded_call/1') :- !.
-user:prolog_predicate_name(user:'$lgt_threaded_call'(_, _, _, _), 'threaded_call/1') :- !.
-user:prolog_predicate_name(user:'$lgt_threaded_once'(_, _, _), 'threaded_once/1') :- !.
-user:prolog_predicate_name(user:'$lgt_threaded_once'(_, _, _, _), 'threaded_once/1') :- !.
-user:prolog_predicate_name(user:'$lgt_threaded_call_tagged'(_, _, _, _), 'threaded_call/2') :- !.
-user:prolog_predicate_name(user:'$lgt_threaded_call_tagged'(_, _, _, _, _), 'threaded_call/2') :- !.
-user:prolog_predicate_name(user:'$lgt_threaded_once_tagged'(_, _, _, _), 'threaded_once/2') :- !.
-user:prolog_predicate_name(user:'$lgt_threaded_once_tagged'(_, _, _, _, _), 'threaded_once/2') :- !.
-user:prolog_predicate_name(user:'$lgt_threaded_peek'(_, _, _, _), 'threaded_peek/1') :- !.
-user:prolog_predicate_name(user:'$lgt_threaded_peek'(_, _, _, _, _), 'threaded_peek/1') :- !.
-user:prolog_predicate_name(user:'$lgt_threaded_peek_tagged'(_, _, _, _, _), 'threaded_peek/2') :- !.
-user:prolog_predicate_name(user:'$lgt_threaded_peek_tagged'(_, _, _, _, _, _), 'threaded_peek/2') :- !.
-user:prolog_predicate_name(user:'$lgt_threaded_exit'(_, _, _, _), 'threaded_exit/1') :- !.
-user:prolog_predicate_name(user:'$lgt_threaded_exit'(_, _, _, _, _), 'threaded_exit/1') :- !.
-user:prolog_predicate_name(user:'$lgt_threaded_exit_tagged'(_, _, _, _, _), 'threaded_exit/2') :- !.
-user:prolog_predicate_name(user:'$lgt_threaded_exit_tagged'(_, _, _, _, _, _), 'threaded_exit/2') :- !.
-user:prolog_predicate_name(user:'$lgt_threaded_wait_synch_ctg'(_, _, _), 'threaded_wait/1') :- !.
-user:prolog_predicate_name(user:'$lgt_threaded_wait_synch'(_, _, _), 'threaded_wait/1') :- !.
-user:prolog_predicate_name(user:'$lgt_threaded_wait_ctg'(_, _), 'threaded_wait/1') :- !.
-user:prolog_predicate_name(user:'$lgt_threaded_wait'(_, _), 'threaded_wait/1') :- !.
-user:prolog_predicate_name(user:'$lgt_threaded_notify_ctg'(_, _), 'threaded_notify/1') :- !.
-user:prolog_predicate_name(user:'$lgt_threaded_notify'(_, _), 'threaded_notify/1') :- !.
+user:prolog_predicate_name(QualifiedInternalName, UserName) :-
+	'$lgt_user_module_qualification'(InternalName, QualifiedInternalName),
+	'$lgt_swi_prolog_predicate_name'(InternalName, UserName).
 
 user:prolog_predicate_name(Goal, Label) :-
 	Goal \= _::_,
+	'$lgt_user_module_qualification'(_, Module:_),
 	(	Goal = Module:THead ->
-		Module == user
+		true
 	;	Goal = THead
 	),
 	functor(THead, TFunctor, TArity),
@@ -198,16 +110,105 @@ user:prolog_predicate_name(Goal, Label) :-
 		atomic_list_concat([EFunctor, '/', EArity, '::', Functor, '/', Arity], Label)
 	).
 
+'$lgt_swi_prolog_predicate_name'('$lgt_send_to_obj_'(_, _, _), '::/2 (event-aware)') :- !.
+'$lgt_swi_prolog_predicate_name'('$lgt_send_to_obj_ne_'(_, _, _), '::/2 (event transparent)') :- !.
+'$lgt_swi_prolog_predicate_name'('$lgt_send_to_self_'(_, _, _), '::/1') :- !.
+'$lgt_swi_prolog_predicate_name'('$lgt_obj_super_call_'(_, _, _), '^^/2 (from obj; same pred)') :- !.
+'$lgt_swi_prolog_predicate_name'('$lgt_ctg_super_call_'(_, _, _), '^^/2 (from ctg; same pred)') :- !.
+'$lgt_swi_prolog_predicate_name'('$lgt_call_in_this'(_, _), 'call/1') :- !.
+
+'$lgt_swi_prolog_predicate_name'('$lgt_send_to_obj_rt'(_, _, _, _), '::/2 (runtime)') :- !.
+'$lgt_swi_prolog_predicate_name'('$lgt_send_to_obj'(_, _, _), '::/2 (event transparent)') :- !.
+'$lgt_swi_prolog_predicate_name'('$lgt_send_to_obj_ne'(_, _, _), '::/2 (event transparent)') :- !.
+'$lgt_swi_prolog_predicate_name'('$lgt_send_to_self'(_, _, _), '::/1') :- !.
+'$lgt_swi_prolog_predicate_name'('$lgt_obj_super_call'(_, _, _), '^^/2 (from obj; same pred)') :- !.
+'$lgt_swi_prolog_predicate_name'('$lgt_ctg_super_call'(_, _, _), '^^/2 (from ctg; same pred)') :- !.
+
+'$lgt_swi_prolog_predicate_name'('$lgt_metacall'(_, _, _), 'call/N') :- !.
+'$lgt_swi_prolog_predicate_name'('$lgt_metacall'(_, _), 'call/1') :- !.
+'$lgt_swi_prolog_predicate_name'('$lgt_quantified_metacall'(_, _, _), 'call/1') :- !.
+'$lgt_swi_prolog_predicate_name'('$lgt_metacall_local'(_, _), 'call/1') :- !.
+'$lgt_swi_prolog_predicate_name'('$lgt_metacall_sender'(_, _, _, _), 'call/1') :- !.
+
+'$lgt_swi_prolog_predicate_name'('$lgt_bagof'(_, _, _, _, _), 'bagof/3') :- !.
+'$lgt_swi_prolog_predicate_name'('$lgt_setof'(_, _, _, _, _), 'setof/3') :- !.
+
+'$lgt_swi_prolog_predicate_name'('$lgt_expand_term_local'(_, _, _, _), 'expand_term/2') :- !.
+'$lgt_swi_prolog_predicate_name'('$lgt_expand_term_message'(_, _, _, _, _), 'expand_term/2') :- !.
+'$lgt_swi_prolog_predicate_name'('$lgt_expand_goal_local'(_, _, _, _), 'expand_goal/2') :- !.
+'$lgt_swi_prolog_predicate_name'('$lgt_expand_goal_message'(_, _, _, _, _), 'expand_goal/2') :- !.
+
+'$lgt_swi_prolog_predicate_name'('$lgt_phrase'(_, _, _), 'phrase/2') :- !.
+'$lgt_swi_prolog_predicate_name'('$lgt_phrase'(_, _, _, _), 'phrase/3') :- !.
+
+'$lgt_swi_prolog_predicate_name'('$lgt_compiler_flag'(_, _), 'current_logtalk_flag/2') :- !.
+'$lgt_swi_prolog_predicate_name'('$lgt_set_compiler_flag'(_, _), 'set_logtalk_flag/2') :- !.
+
+'$lgt_swi_prolog_predicate_name'('$lgt_current_op'(_, _, _, _, _, _), 'current_op/3') :- !.
+'$lgt_swi_prolog_predicate_name'('$lgt_current_predicate'(_, _, _, _), 'current_predicate/1') :- !.
+'$lgt_swi_prolog_predicate_name'('$lgt_predicate_property'(_, _, _, _, _), 'predicate_property/2') :- !.
+
+'$lgt_swi_prolog_predicate_name'('$lgt_abolish_checked'(_, _, _, _), 'abolish/1') :- !.
+'$lgt_swi_prolog_predicate_name'('$lgt_asserta_fact_checked'(_, _, _, _, _), 'asserta/1') :- !.
+'$lgt_swi_prolog_predicate_name'('$lgt_asserta_rule_checked'(_, _, _, _, _), 'asserta/1') :- !.
+'$lgt_swi_prolog_predicate_name'('$lgt_assertz_fact_checked'(_, _, _, _, _), 'assertz/1') :- !.
+'$lgt_swi_prolog_predicate_name'('$lgt_assertz_rule_checked'(_, _, _, _, _), 'assertz/1') :- !.
+'$lgt_swi_prolog_predicate_name'('$lgt_clause_checked'(_, _, _, _, _), 'clause/2') :- !.
+'$lgt_swi_prolog_predicate_name'('$lgt_retract_fact_checked'(_, _, _, _), 'retract/1') :- !.
+'$lgt_swi_prolog_predicate_name'('$lgt_retract_rule_checked'(_, _, _, _), 'retract/1') :- !.
+'$lgt_swi_prolog_predicate_name'('$lgt_retractall_checked'(_, _, _, _), 'retractall/1') :- !.
+
+'$lgt_swi_prolog_predicate_name'('$lgt_iso_read_term'(_, _, _, _), 'read_term/3') :- !.
+'$lgt_swi_prolog_predicate_name'('$lgt_iso_read_term'(_, _, _), 'read_term/2') :- !.
+'$lgt_swi_prolog_predicate_name'('$lgt_iso_read'(_, _, _), 'read/2') :- !.
+'$lgt_swi_prolog_predicate_name'('$lgt_iso_read'(_, _), 'read/1') :- !.
+
+'$lgt_swi_prolog_predicate_name'('$lgt_iso_write_term'(_, _, _, _), 'write_term/3') :- !.
+'$lgt_swi_prolog_predicate_name'('$lgt_iso_write_term'(_, _, _), 'write_term/2') :- !.
+'$lgt_swi_prolog_predicate_name'('$lgt_iso_write'(_, _, _), 'write/2') :- !.
+'$lgt_swi_prolog_predicate_name'('$lgt_iso_write'(_, _), 'write/1') :- !.
+'$lgt_swi_prolog_predicate_name'('$lgt_iso_writeq'(_, _, _), 'writeq/2') :- !.
+'$lgt_swi_prolog_predicate_name'('$lgt_iso_writeq'(_, _), 'writeq/1') :- !.
+
+'$lgt_swi_prolog_predicate_name'('$lgt_category_parameter'(_, _, _, _), 'parameter/2') :- !.
+
+'$lgt_swi_prolog_predicate_name'('$lgt_threaded_or'(_, _, _), 'threaded/1') :- !.
+'$lgt_swi_prolog_predicate_name'('$lgt_threaded_and'(_, _, _), 'threaded/1') :- !.
+'$lgt_swi_prolog_predicate_name'('$lgt_threaded_ignore'(_), 'threaded_ignore/1') :- !.
+'$lgt_swi_prolog_predicate_name'('$lgt_threaded_call'(_, _, _), 'threaded_call/1') :- !.
+'$lgt_swi_prolog_predicate_name'('$lgt_threaded_call'(_, _, _, _), 'threaded_call/1') :- !.
+'$lgt_swi_prolog_predicate_name'('$lgt_threaded_once'(_, _, _), 'threaded_once/1') :- !.
+'$lgt_swi_prolog_predicate_name'('$lgt_threaded_once'(_, _, _, _), 'threaded_once/1') :- !.
+'$lgt_swi_prolog_predicate_name'('$lgt_threaded_call_tagged'(_, _, _, _), 'threaded_call/2') :- !.
+'$lgt_swi_prolog_predicate_name'('$lgt_threaded_call_tagged'(_, _, _, _, _), 'threaded_call/2') :- !.
+'$lgt_swi_prolog_predicate_name'('$lgt_threaded_once_tagged'(_, _, _, _), 'threaded_once/2') :- !.
+'$lgt_swi_prolog_predicate_name'('$lgt_threaded_once_tagged'(_, _, _, _, _), 'threaded_once/2') :- !.
+'$lgt_swi_prolog_predicate_name'('$lgt_threaded_peek'(_, _, _, _), 'threaded_peek/1') :- !.
+'$lgt_swi_prolog_predicate_name'('$lgt_threaded_peek'(_, _, _, _, _), 'threaded_peek/1') :- !.
+'$lgt_swi_prolog_predicate_name'('$lgt_threaded_peek_tagged'(_, _, _, _, _), 'threaded_peek/2') :- !.
+'$lgt_swi_prolog_predicate_name'('$lgt_threaded_peek_tagged'(_, _, _, _, _, _), 'threaded_peek/2') :- !.
+'$lgt_swi_prolog_predicate_name'('$lgt_threaded_exit'(_, _, _, _), 'threaded_exit/1') :- !.
+'$lgt_swi_prolog_predicate_name'('$lgt_threaded_exit'(_, _, _, _, _), 'threaded_exit/1') :- !.
+'$lgt_swi_prolog_predicate_name'('$lgt_threaded_exit_tagged'(_, _, _, _, _), 'threaded_exit/2') :- !.
+'$lgt_swi_prolog_predicate_name'('$lgt_threaded_exit_tagged'(_, _, _, _, _, _), 'threaded_exit/2') :- !.
+'$lgt_swi_prolog_predicate_name'('$lgt_threaded_wait_synch_ctg'(_, _, _), 'threaded_wait/1') :- !.
+'$lgt_swi_prolog_predicate_name'('$lgt_threaded_wait_synch'(_, _, _), 'threaded_wait/1') :- !.
+'$lgt_swi_prolog_predicate_name'('$lgt_threaded_wait_ctg'(_, _), 'threaded_wait/1') :- !.
+'$lgt_swi_prolog_predicate_name'('$lgt_threaded_wait'(_, _), 'threaded_wait/1') :- !.
+'$lgt_swi_prolog_predicate_name'('$lgt_threaded_notify_ctg'(_, _), 'threaded_notify/1') :- !.
+'$lgt_swi_prolog_predicate_name'('$lgt_threaded_notify'(_, _), 'threaded_notify/1') :- !.
+
 
 :- multifile(prolog:term_compiled/2).
 
 prolog:term_compiled(Entity::Head, QHead) :-
+	'$lgt_user_module_qualification'(_, Module:_),
 	(	callable(Entity), callable(Head) ->
 		'$lgt_compile_predicate_heads'(Head, Entity, THead, _),
-		QHead = user:THead
+		QHead = Module:THead
 	;	callable(QHead) ->
 		(	QHead = Module:THead ->
-			Module == user
+			true
 		;	QHead = THead
 		),
 		'$lgt_decompile_predicate_heads'(THead, Entity, _, Head)
@@ -218,13 +219,12 @@ prolog:term_compiled(Entity::Head, QHead) :-
 :- multifile(prolog_clause:unify_clause_hook/5).
 
 prolog_clause:unify_clause_hook(Clause, QClause, Module, TermPos0, TermPos) :-
-	(	QClause = (M:THead :- TBody) ->
-		M == user,
+	'$lgt_user_module_qualification'(_, Module:_),
+	(	QClause = (Module:THead :- TBody) ->
 		TClause = (THead :- TBody)
 	;	QClause = (THead :- _) ->
 		TClause = QClause
-	;	QClause = M:THead ->
-		M == user,
+	;	QClause = Module:THead ->
 		TClause = THead
 	;	QClause = THead ->
 		TClause = QClause
@@ -250,13 +250,14 @@ prolog_clause:unify_clause_hook(Clause, QClause, Module, TermPos0, TermPos) :-
 
 :- multifile(prolog_clause:make_varnames_hook/5).
 
-prolog_clause:make_varnames_hook((Head --> _), (user:THead :- _), Offsets, Names, Bindings) :-
+prolog_clause:make_varnames_hook((Head --> _), (Module:THead :- _), Offsets, Names, Bindings) :-
+	'$lgt_user_module_qualification'(_, Module:_),
 	functor(THead, TFunctor, THeadArity),
 	'$lgt_current_flag_'(code_prefix, CodePrefix),
 	atom_concat(CodePrefix, _, TFunctor),
 	N is THeadArity - 1,
 	memberchk(N=EVar, Offsets),
-	Names1 = ['<Sender, This, Self, MetaVars, CoinductionStack>'=EVar| Names],
+	Names1 = ['<Entity, Sender, This, Self, MetaVars, CoinductionStack>'=EVar| Names],
 	functor(Head, _, HeadArity),
 	In is HeadArity,
 	memberchk(In=IVar, Offsets),
@@ -265,13 +266,14 @@ prolog_clause:make_varnames_hook((Head --> _), (user:THead :- _), Offsets, Names
 	memberchk(Out=OVar, Offsets),
 	Names3 = ['<DCG_tail>'=OVar|Names2],
 	prolog_clause:make_varnames(xx, xx, Offsets, Names3, Bindings).
-prolog_clause:make_varnames_hook(_, (user:THead :- _), Offsets, Names, Bindings) :-
+prolog_clause:make_varnames_hook(_, (Module:THead :- _), Offsets, Names, Bindings) :-
+	'$lgt_user_module_qualification'(_, Module:_),
 	functor(THead, TFunctor, Arity),
 	'$lgt_current_flag_'(code_prefix, CodePrefix),
 	atom_concat(CodePrefix, _, TFunctor),
 	N is Arity - 1,
 	memberchk(N=IVar, Offsets),
-	Names1 = ['<Sender, This, Self, MetaVars, CoinductionStack>'=IVar| Names],
+	Names1 = ['<Entity, Sender, This, Self, MetaVars, CoinductionStack>'=IVar| Names],
 	prolog_clause:make_varnames(xx, xx, Offsets, Names1, Bindings).
 
 
@@ -359,7 +361,7 @@ user:portray(c(This, Entity, Rest)) :-
 	!,
 	length(ExtraArgs, N),
 	Arity is N + 1,
-	catch(arg(1, CallN, Closure), Error, (writeln('ERROR 2'-Error), throw(Error))),
+	arg(1, CallN, Closure),
 	'$lgt_swi_call_n_args'(ExtraArgs, 2, CallN).
 '$lgt_swi_unify_clause_body'(Goal, _, '$lgt_metacall'(Goal, _), TermPos, TermPos) :- !.
 '$lgt_swi_unify_clause_body'(Goal, _, '$lgt_quantified_metacall'(Goal, _, _), TermPos, TermPos) :- !.
@@ -565,7 +567,7 @@ user:portray(c(This, Entity, Rest)) :-
 
 '$lgt_swi_call_n_args'([], _, _).
 '$lgt_swi_call_n_args'([Arg| Args], N, CallN) :-
-	catch(arg(N, CallN, Arg), Error, (writeln('ERROR 1'-Error), throw(Error))),
+	arg(N, CallN, Arg),
 	N2 is N + 1,
 	'$lgt_swi_call_n_args'(Args, N2, CallN).
 
@@ -655,85 +657,85 @@ user:portray(c(This, Entity, Rest)) :-
 % add dummy meta_predicate/1 directives to avoid cluttering the make/0
 % analysis report (as some of the results are not correct for all usage
 % cases and can lead to false warnings)
-:- meta_predicate user:'$lgt_ctg_super_call_nv'(*,*,*).
-:- meta_predicate user:'$lgt_obj_super_call_nv'(*,*,*).
-:- meta_predicate user:'$lgt_category_property_declares'(*,*,*,*).
-:- meta_predicate user:'$lgt_category_property'(*,*,*,*,*,*).
-:- meta_predicate user:'$lgt_send_to_obj_nv_inner'(*,*,*,*).
-:- meta_predicate user:'$lgt_object_property_declares'(*,*,*,*,*,*).
-:- meta_predicate user:'$lgt_object_property_defines'(*,*,*,*,*).
-:- meta_predicate user:'$lgt_find_original_predicate'(*,*,*,*,*,*).
-:- meta_predicate user:'$lgt_find_original_predicate'(*,*,*,*,*,*,*).
-:- meta_predicate user:'$lgt_entity_property_alias'(*,*,*,*,*).
-:- meta_predicate user:'$lgt_expand_goal_category_scoped'(*,*,*,*).
-:- meta_predicate user:'$lgt_expand_goal_category_local'(*,*,*,*).
-:- meta_predicate user:'$lgt_expand_goal_object_scoped'(*,*,*,*).
-:- meta_predicate user:'$lgt_expand_goal_object_local'(*,*,*,*,*).
-:- meta_predicate user:'$lgt_expand_goal_message'(*,*,*,*).
-:- meta_predicate user:'$lgt_assert_pred_dcl'(*,*,*,*,*,*,*,*,*,*,*,*,*).
-:- meta_predicate user:'$lgt_define_events'(*,*,*,*,*,*,*).
-:- meta_predicate user:'$lgt_threaded_or'(*,*,*).
-:- meta_predicate user:'$lgt_mt_non_det_goal'(*,*,*,*,*).
-:- meta_predicate user:'$lgt_protocol_property_declares'(*,*,*,*).
-:- meta_predicate user:'$lgt_guarded_method_call'(*,*,*,*).
-:- meta_predicate user:'$lgt_threaded_and'(*,*,*).
-:- meta_predicate user:'$lgt_category_property_defines'(*,*,*,*).
-:- meta_predicate user:'$lgt_protocol_property'(*,*,*,*,*).
-:- meta_predicate user:'$lgt_mt_det_goal'(*,*,*,*,*).
-:- meta_predicate user:'$lgt_abolish_entity_predicates'(*).
-:- meta_predicate user:'$lgt_threaded_once'(*,*,*,*).
-:- meta_predicate user:'$lgt_threaded_call_tagged'(*,*,*,*,*).
-:- meta_predicate user:'$lgt_threaded_call'(*,*,*,*).
-:- meta_predicate user:'$lgt_predicate_property_user'(*,*,*,*,*,*,*,*,*,*,*).
-:- meta_predicate user:'$lgt_threaded_once_tagged'(*,*,*,*,*).
-:- meta_predicate user:'$lgt_send_to_obj_ne_nv'(*,*,*).
-:- meta_predicate user:'$lgt_mt_threaded_call'(*,*,*).
-:- meta_predicate user:'$lgt_threaded_ignore'(*).
-:- meta_predicate user:'$lgt_send_to_obj_nv'(*,*,*).
-:- meta_predicate user:'$lgt_complemented_object'(*,*,*,*,*,*,*,*).
-:- meta_predicate user:'$lgt_complemented_object'(*,*,*,*,*,*).
-:- meta_predicate user:'$lgt_object_property'(*,*,*,*,*,*,*,*).
-:- meta_predicate user:'$lgt_object_property_resources'(*,*,*,*,*,*).
-:- meta_predicate user:'$lgt_object_property_resource'(*,*,*,*,*,*).
-:- meta_predicate user:'$lgt_protocol_property_resources'(*,*,*,*,*).
-:- meta_predicate user:'$lgt_protocol_property_resource'(*,*,*,*,*).
-:- meta_predicate user:'$lgt_category_property_resource'(*,*,*,*,*).
-:- meta_predicate user:'$lgt_category_property_resources'(*,*,*,*,*).
+:- meta_predicate '$lgt_ctg_super_call_nv'(*,*,*).
+:- meta_predicate '$lgt_obj_super_call_nv'(*,*,*).
+:- meta_predicate '$lgt_category_property_declares'(*,*,*,*).
+:- meta_predicate '$lgt_category_property'(*,*,*,*,*,*).
+:- meta_predicate '$lgt_send_to_obj_nv_inner'(*,*,*,*).
+:- meta_predicate '$lgt_object_property_declares'(*,*,*,*,*,*).
+:- meta_predicate '$lgt_object_property_defines'(*,*,*,*,*).
+:- meta_predicate '$lgt_find_original_predicate'(*,*,*,*,*,*).
+:- meta_predicate '$lgt_find_original_predicate'(*,*,*,*,*,*,*).
+:- meta_predicate '$lgt_entity_property_alias'(*,*,*,*,*).
+:- meta_predicate '$lgt_expand_goal_category_scoped'(*,*,*,*).
+:- meta_predicate '$lgt_expand_goal_category_local'(*,*,*,*).
+:- meta_predicate '$lgt_expand_goal_object_scoped'(*,*,*,*).
+:- meta_predicate '$lgt_expand_goal_object_local'(*,*,*,*,*).
+:- meta_predicate '$lgt_expand_goal_message'(*,*,*,*).
+:- meta_predicate '$lgt_assert_pred_dcl'(*,*,*,*,*,*,*,*,*,*,*,*,*).
+:- meta_predicate '$lgt_define_events'(*,*,*,*,*,*,*).
+:- meta_predicate '$lgt_threaded_or'(*,*,*).
+:- meta_predicate '$lgt_mt_non_det_goal'(*,*,*,*,*).
+:- meta_predicate '$lgt_protocol_property_declares'(*,*,*,*).
+:- meta_predicate '$lgt_guarded_method_call'(*,*,*,*).
+:- meta_predicate '$lgt_threaded_and'(*,*,*).
+:- meta_predicate '$lgt_category_property_defines'(*,*,*,*).
+:- meta_predicate '$lgt_protocol_property'(*,*,*,*,*).
+:- meta_predicate '$lgt_mt_det_goal'(*,*,*,*,*).
+:- meta_predicate '$lgt_abolish_entity_predicates'(*).
+:- meta_predicate '$lgt_threaded_once'(*,*,*,*).
+:- meta_predicate '$lgt_threaded_call_tagged'(*,*,*,*,*).
+:- meta_predicate '$lgt_threaded_call'(*,*,*,*).
+:- meta_predicate '$lgt_predicate_property_user'(*,*,*,*,*,*,*,*,*,*,*).
+:- meta_predicate '$lgt_threaded_once_tagged'(*,*,*,*,*).
+:- meta_predicate '$lgt_send_to_obj_ne_nv'(*,*,*).
+:- meta_predicate '$lgt_mt_threaded_call'(*,*,*).
+:- meta_predicate '$lgt_threaded_ignore'(*).
+:- meta_predicate '$lgt_send_to_obj_nv'(*,*,*).
+:- meta_predicate '$lgt_complemented_object'(*,*,*,*,*,*,*,*).
+:- meta_predicate '$lgt_complemented_object'(*,*,*,*,*,*).
+:- meta_predicate '$lgt_object_property'(*,*,*,*,*,*,*,*).
+:- meta_predicate '$lgt_object_property_resources'(*,*,*,*,*,*).
+:- meta_predicate '$lgt_object_property_resource'(*,*,*,*,*,*).
+:- meta_predicate '$lgt_protocol_property_resources'(*,*,*,*,*).
+:- meta_predicate '$lgt_protocol_property_resource'(*,*,*,*,*).
+:- meta_predicate '$lgt_category_property_resource'(*,*,*,*,*).
+:- meta_predicate '$lgt_category_property_resources'(*,*,*,*,*).
 
-:- meta_predicate user:'$lgt_assert_pred_def'(*,*,*,*,*,*,*,*).
-:- meta_predicate user:'$lgt_send_to_self_nv'(*,*,*).
+:- meta_predicate '$lgt_assert_pred_def'(*,*,*,*,*,*,*,*).
+:- meta_predicate '$lgt_send_to_self_nv'(*,*,*).
 
-:- meta_predicate user:threaded_ignore(*).
-:- meta_predicate user:threaded_once(*,*).
-:- meta_predicate user:threaded_call(*,*).
-:- meta_predicate user:threaded_call(*).
-:- meta_predicate user:threaded_once(*).
+:- meta_predicate threaded_ignore(*).
+:- meta_predicate threaded_once(*,*).
+:- meta_predicate threaded_call(*,*).
+:- meta_predicate threaded_call(*).
+:- meta_predicate threaded_once(*).
 
-:- meta_predicate user:'$lgt_threaded_goal'(*,*,*,*).
-:- meta_predicate user:'$lgt_threaded_call'(*,*,*).
-:- meta_predicate user:'$lgt_threaded_once'(*,*,*).
-:- meta_predicate user:'$lgt_ctg_super_call_'(*,*,*).
-:- meta_predicate user:'$lgt_send_to_obj_'(*,*,*).
-:- meta_predicate user:'$lgt_obj_super_call_'(*,*,*).
-:- meta_predicate user:'$lgt_send_to_obj_ne_'(*,*,*).
-:- meta_predicate user:'$lgt_threaded_once_tagged'(*,*,*,*).
-:- meta_predicate user:'$lgt_send_to_self_'(*,*,*).
-:- meta_predicate user:'$lgt_threaded_call_tagged'(*,*,*,*).
+:- meta_predicate '$lgt_threaded_goal'(*,*,*,*).
+:- meta_predicate '$lgt_threaded_call'(*,*,*).
+:- meta_predicate '$lgt_threaded_once'(*,*,*).
+:- meta_predicate '$lgt_ctg_super_call_'(*,*,*).
+:- meta_predicate '$lgt_send_to_obj_'(*,*,*).
+:- meta_predicate '$lgt_obj_super_call_'(*,*,*).
+:- meta_predicate '$lgt_send_to_obj_ne_'(*,*,*).
+:- meta_predicate '$lgt_threaded_once_tagged'(*,*,*,*).
+:- meta_predicate '$lgt_send_to_self_'(*,*,*).
+:- meta_predicate '$lgt_threaded_call_tagged'(*,*,*,*).
 
-:- meta_predicate user:'$lgt_send_to_obj_rt'(*,*,*,*).
-:- meta_predicate user:'$lgt_send_to_obj_ne'(*,*,*).
-:- meta_predicate user:'$lgt_ctg_super_call'(*,*,*).
-:- meta_predicate user:'$lgt_obj_super_call'(*,*,*).
-:- meta_predicate user:'$lgt_send_to_obj'(*,*,*).
-:- meta_predicate user:'$lgt_send_to_self'(*,*,*).
-:- meta_predicate user:'$lgt_call_proxy'(*,*,*).
-:- meta_predicate user:'$lgt_call_within_context_nv'(*,*,*).
-:- meta_predicate user:'$lgt_call_within_context'(*,*,*).
+:- meta_predicate '$lgt_send_to_obj_rt'(*,*,*,*).
+:- meta_predicate '$lgt_send_to_obj_ne'(*,*,*).
+:- meta_predicate '$lgt_ctg_super_call'(*,*,*).
+:- meta_predicate '$lgt_obj_super_call'(*,*,*).
+:- meta_predicate '$lgt_send_to_obj'(*,*,*).
+:- meta_predicate '$lgt_send_to_self'(*,*,*).
+:- meta_predicate '$lgt_call_proxy'(*,*,*).
+:- meta_predicate '$lgt_call_within_context_nv'(*,*,*).
+:- meta_predicate '$lgt_call_within_context'(*,*,*).
 
-:- meta_predicate user:'$lgt_metacall'(*,*).
-:- meta_predicate user:'$lgt_metacall'(*,*,*).
-:- meta_predicate user:'$lgt_quantified_metacall'(*,*,*).
-:- meta_predicate user:'$lgt_metacall_sender'(*,*,*,*).
-:- meta_predicate user:'$lgt_metacall_local'(*,*).
+:- meta_predicate '$lgt_metacall'(*,*).
+:- meta_predicate '$lgt_metacall'(*,*,*).
+:- meta_predicate '$lgt_quantified_metacall'(*,*,*).
+:- meta_predicate '$lgt_metacall_sender'(*,*,*,*).
+:- meta_predicate '$lgt_metacall_local'(*,*).
 
-:- meta_predicate user:'$user#0.forward#1'(*,*).
+:- meta_predicate '$user#0.forward#1'(*,*).
